==> cyber-orm-extras-bundle-7.3.1-b58931.zip: src/Spec/EntitySpecRepoTrait.php <==
<?php

/**
 * This file is subject to the terms and conditions defined in file 'LICENSE', which is part of this source code
 * package. If the file is missing a copy can be found at:
 * https://gitlab.cybercoder.site/vj/policies-procedures-standards/blob/master/licensing/CYBER-LICENSE.
 */

declare(strict_types=1);

namespace Cyber\OrmExtras\Spec;

use Cyber\OrmExtras\Spec;
use Doctrine\ORM\AbstractQuery;
use Doctrine\ORM\NonUniqueResultException;
use Doctrine\ORM\QueryBuilder;
use InvalidArgumentException;

/**
 * @method string       getEntityName()
 * @method QueryBuilder createQueryBuilder(string $alias)
 *
 * @property string $defaultAlias
 */
trait EntitySpecRepoTrait
{
    /**
     * @param Spec $specification
     *
     * @throws NonUniqueResultException
     *
     * @return mixed
     */
    public function matchSpec(Spec $specification)
    {
        $query  = $this->matchSpecQuery($specification);

        return (array) $query->execute();
    }

    /**
     * @param Spec $specification
     *
     * @throws NonUniqueResultException
     *
     * @return mixed
     */
    public function matchSingleSpec(Spec $specification)
    {
        $query  = $this->matchSpecQuery($specification);
        $result = (array) $query->execute();

        $first = \array_shift($result);
        if (!empty($result)) {
            throw new NonUniqueResultException('Single result requested, but multiple rows were returned');
        }

        return $first;
    }

    public function matchSpecQuery(Spec $specification): AbstractQuery
    {
        /** @var class-string $entityClass */
        $entityClass = $this->getEntityName();

        if (!$specification->supports($entityClass)) {
            throw new InvalidArgumentException(\sprintf(
                '%s does not support %s in %s.',
                \get_class($specification),
                $entityClass,
                static::class
            ));
        }

        $qb   = $this->createQueryBuilder($this->getDefaultAlias());
        $expr = $specification->match($qb, $this->getDefaultAlias());
        if (null !== $expr) {
            // maybe spec only does modify query, so it is fine for it not return an expression
            // avoids pollution with 1=1 conditions
            $qb->andWhere($expr);
        }
        $query = $qb->getQuery();

        $specification->modifyQuery($query);

        return $query;
    }

    protected function getDefaultAlias(): string
    {
        return \property_exists($this, 'defaultAlias') ? $this->defaultAlias : 'spec';
    }
}

==> cyber-orm-extras-bundle-7.4.0-808422.zip: src/Spec/EntitySpecRepoTrait.php <==
<?php

/**
 * This file is subject to the terms and conditions defined in file 'LICENSE', which is part of this source code
 * package. If the file is missing a copy can be found at:
 * https://gitlab.cybercoder.site/vj/policies-procedures-standards/blob/master/licensing/CYBER-LICENSE.
 */

declare(strict_types=1);

namespace Cyber\OrmExtras\Spec;

use Cyber\OrmExtras\Spec;
use Doctrine\ORM\AbstractQuery;
use Doctrine\ORM\NonUniqueResultException;
use Doctrine\ORM\QueryBuilder;
use InvalidArgumentException;

/**
 * @method string       getEntityName()
 * @method QueryBuilder createQueryBuilder(string $alias)
 *
 * @property string $defaultAlias
 */
trait EntitySpecRepoTrait
{
    /**
     * @param Spec $specification
     *
     * @throws NonUniqueResultException
     *
     * @return mixed
     */
    public function matchSpec(Spec $specification)
    {
        $query  = $this->matchSpecQuery($specification);

        return (array) $query->execute();
    }

    /**
     * @param Spec $specification
     *
     * @throws NonUniqueResultException
     *
     * @return mixed
     */
    public function matchSingleSpec(Spec $specification)
    {
        $query  = $this->matchSpecQuery($specification);
        $result = (array) $query->execute();

        $first = \array_shift($result);
        if (!empty($result)) {
            throw new NonUniqueResultException('Single result requested, but multiple rows were returned');
        }

        return $first;
    }

    /**
     * Sometimes it is useful to have the query builder instead of the query when matching the spec.
     *
     * Returns QueryBuilder after applying all the spec conditions. At this point modify query has not been executed as there is no query yet.
     */
    public function matchSpecQueryBuilder(Spec $specification): QueryBuilder
    {
        /** @var class-string $entityClass */
        $entityClass = $this->getEntityName();

        if (!$specification->supports($entityClass)) {
            throw new InvalidArgumentException(\sprintf(
                '%s does not support %s in %s.',
                \get_class($specification),
                $entityClass,
                static::class
            ));
        }

        $qb   = $this->createQueryBuilder($this->getDefaultAlias());
        $expr = $specification->match($qb, $this->getDefaultAlias());
        if (null !== $expr) {
            // maybe spec only does modify query, so it is fine for it not return an expression
            // avoids pollution with 1=1 conditions
            $qb->andWhere($expr);
        }

        return $qb;
    }

    public function matchSpecQuery(Spec $specification): AbstractQuery
    {
        $qb    = $this->matchSpecQueryBuilder($specification);
        $query = $qb->getQuery();

        $specification->modifyQuery($query);

        return $query;
    }

    protected function getDefaultAlias(): string
    {
        return \property_exists($this, 'defaultAlias') ? $this->defaultAlias : 'spec';
    }
}
